@@ -120,7 +120,19 @@ calcBasisIntegrals <- function(basisFunctions, dimSupp, argvals)
 #' object, i.e. having one entry for each element of the multivariate functional
 #' data. For each element, \code{uniExpansion} must specify at least the type of
 #' basis functions to use. Additionally, one may add further parameters. The 
-#' following basis representations are supported: \itemize{ \item Univariate 
+#' following basis representations are supported: \itemize{\item Given basis functions. Then \code{uniExpansions[[j]] = 
+#' list(type = "given", functions, scores, ortho)}, where \code{functions} is
+#' a \code{funData} object on the same domain as \code{mFData}, containing 
+#' the given basis functions. The parameters \code{scores} and \code{ortho} 
+#' are optional. \code{scores} is an \code{N x K} matrix containing the 
+#' scores (or coefficients) of the observed functions for the given basis 
+#' functions, where \code{N} is the number of observed functions and \code{K}
+#' is the number of basis functions. If this is not supplied, the scores are 
+#' calculated. The parameter \code{ortho} specifies whether the given basis 
+#' functions are orthonormal \code{orhto = TRUE} or not \code{ortho = FALSE}.
+#' If \code{ortho} is not supplied, the functions are treated as 
+#' non-orthogonal. \code{scores} and \code{ortho} are not checked for
+#' plausibility, use them on your own risk!  \item Univariate 
 #' functional principal component analysis. Then \code{uniExpansions[[j]] = 
 #' list(type = "uFPCA", nbasis, pve, npc, makePD)}, where 
 #' \code{nbasis,pve,npc,makePD} are parameters passed to the \code{\link{PACE}} 
@@ -180,18 +192,19 @@ calcBasisIntegrals <- function(basisFunctions, dimSupp, argvals)
 #'   extra-information about the progress (incl. timestamps). Defaults to 
 #'   \code{options()$verbose}.
 #'   
-#' @return \item{values}{A vector of estimated eigenvalues \eqn{\hat \nu_1 , 
-#'   \ldots , \hat \nu_M}.} \item{functions}{A 
+#' @return An object of class \code{MFPCAfit} containing the following
+#'   components: \item{values}{A vector of estimated eigenvalues \eqn{\hat \nu_1
+#'   , \ldots , \hat \nu_M}.} \item{functions}{A 
 #'   \code{\link[funData]{multiFunData}} object containing the estimated 
 #'   multivariate functional principal components \eqn{\hat \psi_1, \ldots, \hat
 #'   \psi_M}.} \item{scores}{ A matrix of dimension \code{N x M} containing the 
-#'   estimated scores \eqn{\hat \rho_{im}}.} \item{vectors}{A matrix
+#'   estimated scores \eqn{\hat \rho_{im}}.} \item{vectors}{A matrix 
 #'   representing the eigenvectors associated with the combined univariate score
-#'   vectors. This might be helpful for calculating predictions.}
-#'   \item{normFactors}{The normalizing factors used for calculating the
-#'   multivariate eigenfunctions and scores. This might be helpful when
-#'   calculation predictions.} \item{meanFunction}{A multivariate functional
-#'   data object, corresponding to the mean function. The MFPCA is applied to
+#'   vectors. This might be helpful for calculating predictions.} 
+#'   \item{normFactors}{The normalizing factors used for calculating the 
+#'   multivariate eigenfunctions and scores. This might be helpful when 
+#'   calculation predictions.} \item{meanFunction}{A multivariate functional 
+#'   data object, corresponding to the mean function. The MFPCA is applied to 
 #'   the de-meaned functions in \code{mFData}.}\item{fit}{A 
 #'   \code{\link[funData]{multiFunData}} object containing estimated 
 #'   trajectories for each observation based on the truncated Karhunen-Loeve 
@@ -208,13 +221,15 @@ calcBasisIntegrals <- function(basisFunctions, dimSupp, argvals)
 #'   
 #' @importFrom foreach %do%
 #'   
-#' @references C. Happ, S. Greven (2016+):Multivariate Functional Principal 
+#' @references C. Happ, S. Greven (2018): Multivariate Functional Principal 
 #'   Component Analysis for Data Observed on Different (Dimensional) Domains. 
-#'   Journal of the American Statistical Association, to appear. DOI: 
+#'   Journal of the American Statistical Association. Advance online publication. DOI: 
 #'   \url{http://dx.doi.org/10.1080/01621459.2016.1273115}
 #'   
 #' @seealso \code{\link[funData]{multiFunData}}, \code{\link{PACE}}, 
-#'   \code{\link{univDecomp}}, \code{\link{univExpansion}}
+#'   \code{\link{univDecomp}}, \code{\link{univExpansion}}, 
+#'   \code{\link[=summary.MFPCAfit]{summary}}, \code{\link[=plot.MFPCAfit]{plot}}, 
+#'   \code{\link[=scoreplot.MFPCAfit]{scoreplot}}
 #'   
 #' @examples
 #' oldPar <- par(no.readonly = TRUE)
@@ -228,12 +243,20 @@ calcBasisIntegrals <- function(basisFunctions, dimSupp, argvals)
 #' # MFPCA based on univariate FPCA
 #' uFPCA <- MFPCA(sim$simData, M = 5, uniExpansions = list(list(type = "uFPCA"),
 #'                                                                   list(type = "uFPCA")))
+
+#' summary(uFPCA)
+#' plot(uFPCA) # plot the eigenfunctions as perturbations of the mean
+#' scoreplot(uFPCA) # plot the scores
 #' 
 #' # MFPCA based on univariate spline expansions
 #' splines <- MFPCA(sim$simData, M = 5, uniExpansions = list(list(type = "splines1D", k = 10),
 #'                                                           list(type = "splines1D", k = 10)),
 #'                  fit = TRUE) # calculate reconstruction, too
+#' summary(splines)
+#' plot(splines) # plot the eigenfunctions as perturbations of the mean
+#' scoreplot(splines) # plot the scores
 #' 
+#' ### Compare estimates to true eigenfunctions
 #' # flip to make results more clear
 #' uFPCA$functions <- flipFuns(sim$trueFuns, uFPCA$functions)
 #' splines$functions <- flipFuns(sim$trueFuns, splines$functions)
@@ -262,7 +285,8 @@ calcBasisIntegrals <- function(basisFunctions, dimSupp, argvals)
 #' splinesBoot <- MFPCA(sim$simData, M = 2, uniExpansions = list(list(type = "splines1D", k = 10),
 #'                                                           list(type = "splines1D", k = 10)),
 #'                  bootstrap = TRUE, nBootstrap = 100, bootstrapAlpha = c(0.05, 0.1), verbose = TRUE)
-#'                  
+#' summary(splinesBoot)
+#'                                  
 #' plot(splinesBoot$functions[[1]], ylim = c(-2,1.5))
 #' plot(splinesBoot$CI$alpha_0.05$lower[[1]], lty = 2, add = TRUE)
 #' plot(splinesBoot$CI$alpha_0.05$upper[[1]], lty = 2, add = TRUE)
@@ -302,7 +326,11 @@ calcBasisIntegrals <- function(basisFunctions, dimSupp, argvals)
 #' pca <- MFPCA(sim$simData, M = 10,
 #'              uniExpansions = list(list(type = "splines2D", k = c(10,12)),
 #'                              list(type = "uFPCA")))
+#' summary(pca)
+#' plot(pca) # plot the eigenfunctions as perturbations of the mean
+#' scoreplot(pca) # plot the scores
 #' 
+#' ### Compare to true eigenfunctions
 #' # flip to make results more clear
 #' pca$functions <- flipFuns(extractObs(sim$trueFuns, obs = 1:10), pca$functions)
 #' 
@@ -323,15 +351,32 @@ MFPCA <- function(mFData, M, uniExpansions, weights = rep(1, length(mFData)), fi
                   bootstrap = FALSE, nBootstrap = NULL, bootstrapAlpha = 0.05, bootstrapStrat = NULL, 
                   verbose = options()$verbose)
 {
+  if(! inherits(mFData, "multiFunData"))
+    stop("Parameter 'mFData' must be passed as a multiFunData object.")
+  
   # number of components
   p <- length(mFData)
-
   # number of observations
   N <- nObs(mFData)
-
-  if(length(uniExpansions) != p)
-    stop("Function MFPCA_multidim: multivariate functional data object and univariate expansions must have the same length!")
-
+  
+  if(!all(is.numeric(M), length(M) == 1, M > 0))
+    stop("Parameter 'M' must be passed as a number > 0.")
+  
+  if(!(is.list(uniExpansions) & length(uniExpansions) == p))
+    stop("Parameter 'uniExpansions' must be passed as a list with the same length as 'mFData'.")
+  
+  if(!(is.numeric(weights) & length(weights) == p))
+    stop("Parameter 'weights' must be passed as a vector with the same length as 'mFData'.")
+  
+  if(!is.logical(fit))
+    stop("Parameter 'fit' must be passed as a logical.")
+  
+  if(!is.logical(approx.eigen))
+    stop("Parameter 'approx.eigen' must be passed as a logical.")
+  
+  if(!is.logical(bootstrap))
+    stop("Parameter 'bootstrap' must be passed as a logical.")
+  
   if(bootstrap)
   {
     if(is.null(nBootstrap))
@@ -350,6 +395,9 @@ MFPCA <- function(mFData, M, uniExpansions, weights = rep(1, length(mFData)), fi
     }
   }
   
+  if(!is.logical(verbose))
+    stop("Parameter 'verbose' must be passed as a logical.")
+  
   # dimension for each component
   dimSupp <- dimSupp(mFData)
   
@@ -411,8 +459,27 @@ MFPCA <- function(mFData, M, uniExpansions, weights = rep(1, length(mFData)), fi
   
   res$meanFunction <- m # return mean function, too
   
+  names(res$functions) <- names(mFData)
+  
   if(fit)
+  {
     res$fit <- m + res$fit # add mean function to fits
+    names(res$fit) <- names(mFData)
+  } 
+  
+  # give correct names
+  namesList <- lapply(mFData, names)
+  if(!all(sapply(namesList, is.null))) 
+  {
+    if(length(unique(namesList)) != 1)
+      warning("Elements have different curve names. Use names of the first element for the results.")
+    
+    row.names(res$scores) <- namesList[[1]]
+    
+    if(fit)
+      for(i in 1:p)
+        names(res$fit[[i]]) <- namesList[[1]]
+  }
 
   # bootstrap for eigenfunctions
   if(bootstrap)
@@ -515,6 +582,8 @@ MFPCA <- function(mFData, M, uniExpansions, weights = rep(1, length(mFData)), fi
 
     res$CI <- CI
   }
+  
+  class(res) <- "MFPCAfit"
 
   return(res)
 }
@@ -601,7 +670,7 @@ calcMFPCA <- function(N, p, Bchol, M, type, weights, npc, argvals, uniBasis, fit
   tmpWeights <- as.matrix(Matrix::crossprod(Z, Z %*%vectors))
   eFunctions <- foreach::foreach(j = 1:p) %do% {
     univExpansion(type = type[j],
-                  scores = 1/sqrt(weights[j] * values) * normFactors * t(tmpWeights[npcCum[j]+1:npc[j],]),
+                  scores = 1/sqrt(weights[j] * values) * normFactors * t(tmpWeights[npcCum[j]+1:npc[j], , drop = FALSE]),
                   argvals = argvals[[j]],
                   functions = uniBasis[[j]]$functions,
                   params = uniBasis[[j]]$settings)
@@ -613,18 +682,9 @@ calcMFPCA <- function(N, p, Bchol, M, type, weights, npc, argvals, uniBasis, fit
               vectors = vectors,
               normFactors = normFactors)
 
+  # calculate truncated Karhunen-Loeve representation (no mean here)
   if(fit)
-  {
-    # calculate truncated Karhunen-Loeve representation
-    fit <- foreach::foreach(j = 1:p) %do% {
-      univExpansion(type = "default", # calculate linear combination of multivariate basis functions
-                    scores = scores,
-                    argvals = argvals[[j]],
-                    functions = eFunctions[[j]],
-                    params = NULL)
-    }
-    res$fit <- multiFunData(fit)
-  }
+    res$fit <- multivExpansion(multiFuns = res$functions, scores = scores)
 
   return(res)
 }
